"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MergeEditorFloatComponents = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const ide_components_1 = require("@opensumi/ide-components");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const editor_module_less_1 = tslib_1.__importDefault(require("../editor.module.less"));
const MergeEditorFloatComponents = ({ resource }) => {
    const aiNativeConfigService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.AINativeConfigService);
    const commandService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.CommandService);
    const commandRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.CommandRegistry);
    const contextKeyService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.IContextKeyService);
    const [isVisiable, setIsVisiable] = (0, react_1.useState)(false);
    const gitMergeChangesSet = new Set(['git.mergeChanges']);
    (0, react_1.useEffect)(() => {
        const run = () => {
            const mergeChanges = contextKeyService.getValue('git.mergeChanges') || [];
            setIsVisiable(mergeChanges.some((value) => value.toString() === resource.uri.toString()));
        };
        const disposed = contextKeyService.onDidChangeContext(({ payload }) => {
            if (payload.affectsSome(gitMergeChangesSet)) {
                run();
            }
        });
        run();
        return () => disposed.dispose();
    }, [resource]);
    const [isAiResolving, setIsAiResolving] = (0, react_1.useState)(false);
    const handleOpenMergeEditor = (0, react_1.useCallback)(async () => {
        const { uri } = resource;
        [ide_core_browser_1.SCM_COMMANDS.GIT_OPEN_MERGE_EDITOR, ide_core_browser_1.SCM_COMMANDS._GIT_OPEN_MERGE_EDITOR].forEach(({ id: command }) => {
            if (commandRegistry.getCommand(command) && commandRegistry.isEnabled(command)) {
                commandService.executeCommand(command, uri);
            }
        });
    }, [resource]);
    const isSupportAiResolve = (0, react_1.useCallback)(() => aiNativeConfigService.capabilities.supportsConflictResolve, [aiNativeConfigService]);
    const handlePrev = () => {
        commandService.tryExecuteCommand('merge-conflict.previous');
    };
    const handleNext = () => {
        commandService.tryExecuteCommand('merge-conflict.next');
    };
    const handleAIResolve = (0, react_1.useCallback)(async () => {
        setIsAiResolving(true);
        if (isAiResolving) {
            await commandService.executeCommand('merge-conflict.ai.all-accept-stop', resource.uri);
        }
        else {
            await commandService.executeCommand('merge-conflict.ai.all-accept', resource.uri);
        }
        setIsAiResolving(false);
    }, [resource, isAiResolving]);
    const handleReset = (0, react_1.useCallback)(() => {
        commandService.executeCommand('merge-conflict.ai.all-reset', resource.uri);
    }, [resource]);
    if (!isVisiable) {
        return null;
    }
    return (react_1.default.createElement("div", { className: editor_module_less_1.default.merge_editor_float_container },
        react_1.default.createElement("div", { id: 'merge.editor.action.button.accept' },
            react_1.default.createElement(ide_components_1.Button, { className: editor_module_less_1.default.merge_conflict_bottom_btn, size: 'large', onClick: handlePrev },
                react_1.default.createElement(ide_components_1.Icon, { icon: 'left' }),
                react_1.default.createElement("span", null, (0, ide_core_browser_1.localize)('mergeEditor.conflict.prev'))),
            react_1.default.createElement(ide_components_1.Button, { className: editor_module_less_1.default.merge_conflict_bottom_btn, size: 'large', onClick: handleNext },
                react_1.default.createElement("span", null, (0, ide_core_browser_1.localize)('mergeEditor.conflict.next')),
                react_1.default.createElement(ide_components_1.Icon, { icon: 'right' }))),
        react_1.default.createElement("span", { className: editor_module_less_1.default.line_vertical }),
        react_1.default.createElement(ide_components_1.Button, { id: 'merge.editor.open.tradition', className: editor_module_less_1.default.merge_conflict_bottom_btn, size: 'large', onClick: handleOpenMergeEditor },
            react_1.default.createElement(ide_components_1.Icon, { icon: 'swap' }),
            react_1.default.createElement("span", null, (0, ide_core_browser_1.localize)('mergeEditor.open.3way'))),
        react_1.default.createElement(ide_components_1.Button, { id: 'merge.editor.rest', className: editor_module_less_1.default.merge_conflict_bottom_btn, size: 'large', onClick: handleReset },
            react_1.default.createElement(ide_components_1.Icon, { icon: 'discard' }),
            react_1.default.createElement("span", null, (0, ide_core_browser_1.localize)('mergeEditor.reset'))),
        isSupportAiResolve() && (react_1.default.createElement(ide_components_1.Button, { id: 'merge.editor.conflict.resolve.all', size: 'large', className: `${editor_module_less_1.default.merge_conflict_bottom_btn} ${editor_module_less_1.default.magic_btn}`, onClick: handleAIResolve }, isAiResolving ? (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(ide_components_1.Icon, { icon: 'circle-pause' }),
            react_1.default.createElement("span", null, (0, ide_core_browser_1.localize)('mergeEditor.conflict.resolve.all.stop')))) : (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(ide_components_1.Icon, { icon: 'magic-wand' }),
            react_1.default.createElement("span", null, (0, ide_core_browser_1.localize)('mergeEditor.conflict.resolve.all'))))))));
};
exports.MergeEditorFloatComponents = MergeEditorFloatComponents;
//# sourceMappingURL=MergeEditorFloatComponents.js.map